# ディレクトリからリソースを検索して登録する

# フィルタする拡張子
EXT_IMG = /(\.png|\.jpg|\.gif)/ # 画像
EXT_SND = /\.mp3/ # サウンド
EXT_FMF = /\.fmf/ # FMF(マップデータ)
EXT_CSV = /\.csv/ # CSVファイル

# 出力フォルダ
OUTPUT_DIR = "./src/framework/resource/"
SOURCE_DIR = "../../../"

# 出力設定
SETTING = {
	# 画像
	"image" => {
		"class" => "ResourceImage",               # クラス名
		"dir"   => "./resource/image/",           # 検索ディレクトリ
		"ext"   => EXT_IMG,                       # フィルタ拡張子
		"type"  => "Bitmap",                      # 型
		"file"  => OUTPUT_DIR+"ResourceImage.as", # 出力ファイル名
		"embed" => "\t\t[Embed(source='%s')]\n"   # Embed
	},
	# サウンド
	"sound" => {
		"class" => "ResourceSound",               # クラス名
		"dir"   => "./resource/sound/",           # 検索ディレクトリ
		"ext"   => EXT_SND,                       # フィルタ拡張子
		"type"  => "Sound",                       # getValueで返却する型
		"file"  => OUTPUT_DIR+"ResourceSound.as", # 出力ファイル名
		"embed" => "\t\t[Embed(source='%s')]\n"   # Embed
	},
	# FMFファイル
	"fmf" => {
		"class" => "ResourceFmf",               # クラス名
		"dir"   => "./resource/fmf/",           # 検索ディレクトリ
		"ext"   => EXT_FMF,                     # フィルタ拡張子
		"type"  => "ByteArray",                 # 型
		"file"  => OUTPUT_DIR+"ResourceFmf.as", # 出力ファイル名
		"embed" => "\t\t[Embed(source='%s', mimeType='application/octet-stream')]\n" # Embed
	},
	# CSVファイル
	"csv" => {
		"class" => "ResourceCsv",               # クラス名
		"dir"   => "./resource/csv/",           # 検索ディレクトリ
		"ext"   => EXT_CSV,                     # フィルタ拡張子
		"type"  => "ByteArray",                 # 型
		"file"  => OUTPUT_DIR+"ResourceCsv.as", # 出力ファイル名
		"embed" => "\t\t[Embed(source='%s', mimeType='application/octet-stream')]\n" # Embed
	},
}

# ファイルリストを取得する
# @param fileList ファイルリスト(戻り値)
# @param path     検索するルートディレクトリ
# @param regEx    フィルタリングする拡張子
def getFileList(fileList, path, regEx)
	Dir::foreach(path) do |v|
		next if v == "." or v == ".."
		
		if path =~ /\/$/
			v = path + v
		else
			v = path + "/" + v
		end
		if FileTest::directory?(v)
			# ディレクトリ
			getFileList(fileList, v, regEx)
		else
			# ファイル
			if v.scan(/#{regEx}$/).size > 0
				fileList.push(v)
			end
		end
	end
end

def num2alphaL(i)
	a = i/100
	b = (i%100)/10
	c = i%10
	
	ret = ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j"]
	
	return "_" + ret[a] + ret[b] + ret[c]
end
def num2alphaU(i)
	a = i/100
	b = (i%100)/10
	c = i%10
	
	ret = ["A", "B", "C", "D", "E", "F", "G", "H", "I", "J"]
	
	return "_" + ret[a] + ret[b] + ret[c]
end

def getEmbedStr(fList, type, embed)
	ret = ""
	fList.each_with_index do |v, i|
		ret += embed%(SOURCE_DIR+v)
		ret += "\t\tprivate static var %s:Class;\n"%num2alphaL(i)
		ret += "\t\tprivate static var %s:#{type};\n"%num2alphaU(i)
	end
	return ret
end
def getLoadStr(fList, dir, ext)
	ret = ""
	fList.each_with_index do |v, i|
		filename = v.gsub(dir, "")
		name = filename.gsub(ext, "")
		valL = num2alphaL(i)
		valU = num2alphaU(i)
		ret += "\t\t\t#{valU} = new #{valL}();\n"
		ret += "\t\t\t_pool[\"#{name}\"] = #{valU};\n"
	end
	return ret
end

def main
	SETTING.keys.each do |v|
		genAs(v)
	end
end

def genAs(type)
	config = SETTING[type]
	fList = []
	getFileList(fList, config["dir"], config["ext"])
	className = config["class"]
	strEmbed  = getEmbedStr(fList, config["type"], config["embed"])
	strLoad   = getLoadStr(fList, config["dir"], config["ext"])
	strType   = config["type"]
	
	strAsFile = <<EOS
// ************************************************** //
// このコードは<reg_res.rb>によって生成されたものです
// ************************************************** //
package framework.resource
{
	import flash.display.Bitmap;
	import flash.media.Sound;
	import flash.utils.ByteArray;
	/**
	 * リソース管理クラス
	 * @author syun
	 */
	public class #{className}
	{
		private static var _pool:Object = {};
		
#{strEmbed}
		/**
		 * コンストラクタ
		 */
		public function #{className}()
		{
			throw new Error("new禁止");
		}
		/**
		 * ロード
		 */
		public static function load():void
		{
#{strLoad}
		}
		/**
		 * 要素の追加
		 * @param	key キー
		 * @param	val 値
		 */
		public static function setValue(key:String, val:#{strType}):void
		{
			_pool[key] = val;
		}
		
		/**
		 * 取得
		 */
		public static function getValue(key:String):#{strType}
		{
			if (_pool.hasOwnProperty(key))
			{
				return _pool[key];
			}
			throw new Error("指定のリソースは存在しません。key=" + key);
		}
		/**
		 * ハッシュを取得する
		 */
		public static function getHash():Object
		{
			return _pool;
		}
	}
}
EOS
	f = open(config["file"], "w")
	f.write strAsFile
	f.close
end


if __FILE__ == $0
    main()
    print "done."
end
