package framework 
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.InterpolationMethod;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Rectangle;
	import framework.device.graphics.FontManager;
	import framework.device.graphics.GraphicManager;
	import framework.device.input.InputManager;
	import framework.device.sound.SoundManager;
	import framework.resource.ResourceImage;
	import framework.resource.ResourceSound;
	import framework.resource.ResourceFmf;
	import framework.resource.ResourceCsv;
	import framework.scene.SceneManager;
	import framework.util.MessageBox;
	
	/**
	 * ゲームのメインとなるクラス。これを継承して使う
	 * @author syun
	 */
	public class Application extends Sprite
	{
		public static var timer:int; // 汎用タイマー
		private var m_Bg:BitmapData = null;
		private var m_Rect:Rectangle = null;
		private var m_Debug:Boolean = false;
		
		private var m_ScreenWidth:uint;
		private var m_ScreenHeight:uint;
		
		private var m_Fps:FpsView;
		/**
		 * コンストラクタ
		 */ 
		public function Application() 
		{
			m_Fps = new FpsView();
			
			if (stage) { initialize(); }
			else { addEventListener(Event.ADDED_TO_STAGE, initialize); }
		}
		
		/**
		 * タイマーを取得
		 * @param	mod 剰余
		 * @param speed 振れ幅
		 * @return タイマーの値
		 */
		public static function getTimer(mod:int, speed:int=1):int
		{
			return (timer * speed) % mod;
		}
		
		/**
		 * 起動時の初期化
		 * @param	e イベント
		 */
		private function initialize(e:Event=null):void
		{
			// イベントリスナーに登録
			removeEventListener(Event.ADDED_TO_STAGE, create);
			addEventListener(Event.ENTER_FRAME, update);
			create();
		}
		
		/**
		 * ゲーム起動時の初期化処理。overrideして実装する
		 */
		protected function create():void
		{
			throw new Error("Application.create()が未実装");
			
		}
		
		/**
		 * 更新
		 * @param	e
		 */
		private function update(e:Event):void
		{
			timer++;
			
			// 画面揺らし設定を初期化
			IToken._g.shakeEnd();
			
			m_Fps.update();
			InputManager.getInstance().beginFrame();
			{
				// シーンの更新
				SceneManager.getCurrent().update();
				
				// シーンの描画
				IToken._g.setColor();
				IToken._g.fillRect2(m_Rect);
				SceneManager.getCurrent().draw();
				
				// フェードの描画
				IToken._g.drawFade();
				
				// メッセージボックス
				MessageBox.update();
				MessageBox.draw();
			}
			InputManager.getInstance().endFrame();
			m_Fps.draw();
			
			// 画面を揺らす
			IToken._g.shakeBegin();
		}
		
		/**
		 * スクリーンの生成
		 * @param	w 幅
		 * @param	h 高さ
		 */
		protected function createScreen(w:uint, h:uint):void
		{
			if (m_Bg)
			{
				throw new Error("複数回のcreateScreenは禁止です");
			}
			
			// デバッグフラグの初期化
			Debug.initialize();
			
			// スクリーンの生成
			m_ScreenWidth = w;
			m_ScreenHeight = h;
			m_Bg = new BitmapData(w, h, false);
			addChild(new Bitmap(m_Bg));
			m_Rect = new Rectangle(0, 0, w, h);
			// リソースのロード
			ResourceImage.load();
			ResourceSound.load();
			ResourceFmf.load();
			ResourceCsv.load();
			
			// トークンにデバイスを登録する
			IToken._g = new GraphicManager(m_Bg, graphics);
			IToken._m = new SoundManager();
			IToken._i = InputManager.getInstance();
			IToken._f = new FontManager(IToken._g);
		}
		
		protected function setMaxPlayer(nPlayer:uint):void
		{
			InputManager.getInstance().init(stage, 2);
		}
		
	}

}