package framework.device.input
{
	import flash.display.Stage;
	import flash.events.MouseEvent;
	import flash.events.KeyboardEvent;
	import joystick.events.JoyClientEvent;
	import joystick.JoyClient;
	/**
	 * 入力管理
	 * @author syun
	 */
	public class InputManager
	{
		private static var m_Instance:InputManager = null;
		private var m_bInit:Boolean = false;
		private var m_Mouse:Array;
		private var m_Keyboard:Array;
		private var m_Pads:Array;
		
		public static const KEYCODE_LEFTARROW:uint = 37;
		public static const KEYCODE_UPARROW:uint = 38;
		public static const KEYCODE_RIGHTARROW:uint = 39;
		public static const KEYCODE_DOWNARROW:uint = 40;
		public static const KEYCODE_Z:uint = 90;
		public static const KEYCODE_X:uint = 88;
		public static const KEYCODE_C:uint = 67;
		public static const KEYCODE_ENTER:uint = 13;
		
		private var m_JoystickStates:Array;
		private var m_JoyClient:JoyClient;
		
		public function InputManager(caller:Function = null)
		{
			if (caller != InputManager.getInstance)
			{
				throw new Error("new禁止");
			}
		}
		
		public static function getInstance():InputManager
		{
			if (m_Instance == null)
			{
				m_Instance = new InputManager(arguments.callee);
			}
			return m_Instance;
		}

		public function init(stage:Stage, player:uint = 1):void
		{
			if(m_bInit == true) { return; }
			
			m_JoystickStates = new Array();
			m_Mouse = new Array();
			m_Keyboard = new Array();
			m_Pads = new Array();
			for(var i:uint = 0; i < player; i++)
			{
				m_JoystickStates.push(null);
				var m:Mouse = new Mouse();
				var k:Keyboard = new Keyboard(i);
				var p:GamePad = new GamePad();
				p.setKeyboard(k);
				p.setMouse(m);
				m_Mouse.push(m);
				m_Keyboard.push(k);
				m_Pads.push(p);
			}
						
			m_JoyClient = new JoyClient();
			
			m_JoyClient.addEventListener(JoyClientEvent.CONNECTED, onJoyClientConnected);
			m_JoyClient.addEventListener(JoyClientEvent.IO_ERROR, onJoyClientError);
			m_JoyClient.addEventListener(JoyClientEvent.SECURITY_ERROR, onJoyClientError);
			m_JoyClient.addEventListener(JoyClientEvent.PACKET_RECEIVED, onJoyClientReceive);
			m_JoyClient.addEventListener(JoyClientEvent.CLOSED, onJoyClientClose);
			
			m_JoyClient.connect();

			stage.addEventListener(KeyboardEvent.KEY_UP, onKeyUp);
			stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyDown);
			
			stage.addEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
			stage.addEventListener(MouseEvent.CLICK, onMouseClick);
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			
		}
		
		/**
		 * フレームの開始に呼ぶ処理
		 */
		public function beginFrame():void
		{
			for(var i:uint = 0; i < m_Keyboard.length; i++)
			{
//				joyClient.getState(i);
			}
		}
		
		/**
		 * フレームの終了時に呼ぶ処理
		 */
		public function endFrame():void
		{
			for (var i:uint = 0; i < m_Keyboard.length; i++)
			{
				m_Mouse[i].endFrame();
				m_Keyboard[i].endFrame();
				m_JoyClient.getState(i);
			}
		}
		
		private function onJoyClientConnected(e:JoyClientEvent):void
		{
			trace("joystick connected");
			m_JoyClient.getDeviceList();
		}
		
		private function onJoyClientError(e:JoyClientEvent):void
		{
			trace("joystick error");
		}
		
		private function onJoyClientReceive(e:JoyClientEvent):void
		{
			if (e.state.deviceNumber < 0 || e.state.deviceNumber >= m_Keyboard.length)
			{
				return;
			}
			var p:GamePad = m_Pads[e.state.deviceNumber];
			p.setJoystickState(e.state);
		}
		
		private function onJoyClientClose(e:JoyClientEvent):void
		{
			trace("joystick close");
		}
		
		private function onKeyDown(e:KeyboardEvent):void
		{
			for (var i:uint = 0; i < m_Keyboard.length;i++){
				m_Keyboard[i].onKeyDown(e.keyCode);
			}
		}
		
		private function onKeyUp(e:KeyboardEvent):void
		{
			for (var i:uint = 0; i < m_Keyboard.length;i++){
				m_Keyboard[i].onKeyUp(e.keyCode);
			}
		}
		
		private function onMouseMove(e:MouseEvent):void
		{
			for (var i:uint = 0; i < m_Mouse.length; i++)
			{
				m_Mouse[i].onMove(e);
			}
		}
		
		private function onMouseClick(e:MouseEvent):void
		{
			for (var i:uint = 0; i < m_Mouse.length; i++)
			{
				m_Mouse[i].onClick(e);
			}
		}
		
		private function onMouseDown(e:MouseEvent):void
		{
			for (var i:uint = 0; i < m_Mouse.length; i++)
			{
				m_Mouse[i].onDown(e);
			}
		}
		
		private function onMouseUp(e:MouseEvent):void
		{
			for (var i:uint = 0; i < m_Mouse.length; i++)
			{
				m_Mouse[i].onUp(e);
			}
		}
		
		public function getPad(player:uint):GamePad
		{
			return m_Pads[player];
		}
		
	}

}