package framework.device.input
{
	/**
	 * キーボードの入力管理
	 * @author syun
	 */
	public class Keyboard
	{
		/**
		 * ボタンの状態
		 */
		private var m_Button:Array;
		/**
		 * ボタンの状態（１フレーム前）
		 */
		private var m_ButtonPrev:Array;
		/**
		 * 方向キーの状態
		 */
		private var m_Arrow:Array;
		/**
		 * 方向キーの状態（１フレーム前）
		 */
		private var m_ArrowPrev:Array;
		
		private var m_AssignButton:Array;
		private var m_AssignArrow:Array;
		
		/**
		 * 方向キー（左）
		 */
		public static const ARROW_LEFT:uint = 0;
		/**
		 * 方向キー（上）
		 */
		public static const ARROW_UP:uint = 1;
		/**
		 * 方向キー（右）
		 */
		public static const ARROW_RIGHT:uint = 2;
		/**
		 * 方向キー（下）
		 */
		public static const ARROW_DOWN:uint = 3;
		
		
		public static const MAX_BUTTON:uint = 6;

		/**
		 * コンストラクタ 
		 * @param	nPlayer プレイヤー数
		 */
		public function Keyboard(nPlayer:uint = 0) 
		{
			m_Button = new Array();
			m_ButtonPrev = new Array();
			m_Arrow = new Array();
			m_ArrowPrev = new Array();
			
			for(var i:uint = 0; i < MAX_BUTTON; i++)
			{
				m_Button.push(false);
				m_ButtonPrev.push(false);
				m_Arrow.push(false);
				m_ArrowPrev.push(false);
			}
			
			switch(nPlayer)
			{
			case 0:
				m_AssignButton = new Array();
				m_AssignButton.push(InputManager.KEYCODE_Z); // 0
				m_AssignButton.push(InputManager.KEYCODE_X); // 1
				m_AssignButton.push(InputManager.KEYCODE_C); // 2
				m_AssignButton.push(InputManager.KEYCODE_ENTER); // 3
				
				m_AssignArrow = new Array(
					InputManager.KEYCODE_LEFTARROW,
					InputManager.KEYCODE_UPARROW,
					InputManager.KEYCODE_RIGHTARROW,
					InputManager.KEYCODE_DOWNARROW
				);
				break;
			default:
				trace(nPlayer + "player no assign");
				break;
			}
		}
		
		public function endFrame():void
		{
			var i:uint;
			for (i = 0; i < MAX_BUTTON; i++)
			{
				m_ButtonPrev[i] = m_Button[i];
			}
			for (i = 0; i < MAX_BUTTON; i++)
			{
				m_ArrowPrev[i] = m_Arrow[i];
			}
		}
		
		public function onKeyDown(keyCode:uint):void
		{
			var i:uint;
			if(m_AssignButton != null)
			{
				for(i = 0; i < m_AssignButton.length; i++)
				{
					if (keyCode == m_AssignButton[i])
					{
						m_Button[i] = true;
					}
				}
			}
			if (m_AssignArrow != null)
			{
				for(i = 0; i < m_AssignArrow.length; i++)
				{
					if (keyCode == m_AssignArrow[i])
					{
						m_Arrow[i] = true;
					}
				}
			}
		}
		
		public function onKeyUp(keyCode:uint):void
		{
			if(m_AssignButton != null)
			{
				for(var i:uint = 0; i < m_AssignButton.length; i++)
				{
					if(keyCode == m_AssignButton[i])
					{
						m_Button[i] = false;
					}
				}
			}
			
			if (m_AssignArrow != null)
			{
				for (i = 0; i < m_AssignArrow.length; i++)
				{
					if (keyCode == m_AssignArrow[i]) { m_Arrow[i] = false; }
				}
			}
		}
		
		public function isPress(index:uint):Boolean
		{
			return m_Button[index];
		}
		
		public function isPush(index:uint):Boolean
		{
			if(m_Button[index] == true && m_ButtonPrev[index] == false)
			{
				return true;
			}
			return false;
		}
		
		public function isPressArrow(index:uint):Boolean
		{
			return m_Arrow[index];
		}
		
		public function isPushArrow(index:uint):Boolean
		{
			if(m_Arrow[index] == true && m_ArrowPrev[index] == false)
			{
				return true;
			}
			return false;
		}
		
		/**
		 * カーソルキーの入力を取得する（X方向)
		 * @return 左の場合「-1」　右の場合「1」
		 */
		public function getX():int
		{
			return (m_Arrow[ARROW_LEFT] ? -1 : 0) + (m_Arrow[ARROW_RIGHT] ? 1 : 0);
		}
		
		/**
		 * カーソルキーの入力を取得する(Y方向)
		 * @return 上の場合「-1」　下の場合「1」
		 */
		public function getY():int
		{
			return (m_Arrow[ARROW_UP] ? -1 : 0) + (m_Arrow[ARROW_DOWN] ? 1 : 0);
		}
	}

}