package framework.token
{
	import flash.display.Shape;
	import flash.geom.Rectangle;
	import framework.Debug;
	import framework.HitRect;
	import framework.IToken;
	import framework.Maths;
	/**
	 * ...
	 * @author syun
	 */
	public class Token extends IToken
	{
		public var exist:Boolean = false;
		public var x:Number = 0;
		public var y:Number = 0;
		public var vx:Number = 0;
		public var vy:Number = 0;
		public var ax:Number = 0;
		public var ay:Number = 0;
		public var deg:Number = 0;
		public var speed:Number = 0;
		
		public var hitRects:Array;
		
		/**
		 * コンストラクタ
		 */
		public function Token() 
		{
			// 特に何もしない
		}
		
		/**
		 * 
		 * @param	x
		 * @param	y
		 * @param	deg
		 * @param	speed
		 * @param	ax
		 * @param	ay
		 */
		public function create(x:Number, y:Number, deg:Number = 0, speed:Number = 0, ax:Number = 0, ay:Number = 0):void
		{
			this.deg = deg;
			this.speed = speed;
			var dx:Number = Maths.cosEx(deg) * speed;
			var dy:Number = -Maths.sinEx(deg) * speed;
			create2(x, y, dx, dy, ax, ay);
		}
		/**
		 * 生成
		 * @param	x
		 * @param	y
		 * @param	vx
		 * @param	vy
		 * @param	ax
		 * @param	ay
		 */
		public function create2(x:Number, y:Number, vx:Number = 0, vy:Number = 0, ax:Number = 0, ay:Number = 0):void
		{
			exist = true;
			this.x = x;
			this.y = y;
			this.vx = vx;
			this.vy = vy;
			this.ax = ax;
			this.ay = ay;
		}
		
		/**
		 * 消滅
		 */
		public function vanish():void
		{
			exist = false;
		}
		
		/**
		 * 画面外に出たかどうか
		 * @param	w 幅
		 * @param	h 高さ
		 * @return 画面外に出たら「true」
		 */
		public function isOut(w:Number=0, h:Number=0):Boolean
		{
			if (x < -w) { return true; }
			if (x - w > Main.WIDTH) { return true; }
			if (y < -h) { return true; }
			if (y - h > Main.HEIGHT) { return true; }
			return false;
		}
		
		/**
		 * 狙い撃ちの角度を取得する
		 * @param	x
		 * @param	y
		 * @return
		 */
		public function getAim(x:Number, y:Number):Number
		{
			var dx:Number = this.x - x;
			var dy:Number = this.y - y;
			return Maths.atan2Ex(dx, dy);
		}
		
		/**
		 * 移動する
		 */
		public function move():void
		{
			vx += ax;
			vy += ay;
			x += vx;
			y += vy;
		}
		
		/**
		 * ヒットチェック
		 * @param	target 当たり判定の対象
		 * @param	alterHitRects
		 * @return ヒットしていた「true」
		 */
		public function isHit(target:Token , alterHitRects:Array = null):Boolean
		{
			var hr:Array = (alterHitRects == null)? hitRects : alterHitRects;
			if (target == null || hr == null || target.hitRects == null)
			{
				return false;
			}
			
			for each(var h:HitRect in hr)
			{
				var hx:int = x + h.x - h.w/2;
				var hy:int = y + h.y - h.h/2;
				var hx2:int = x + h.x + h.w/2;
				var hy2:int = y + h.y + h.h/2;
				
				for each(var t:HitRect in target.hitRects)
				{
					var tx:int = target.x + t.x - t.w/2;
					var ty:int = target.y + t.y - t.h/2;
					var tx2:int = target.x + t.x + t.w/2;
					var ty2:int = target.y + t.y + t.h/2;
	
					if (hx < tx2 && hx2 > tx && hy < ty2 && hy2 > ty)
					{
						return true;
					}
				}
			}
			return false;
		}
		
		/**
		 * 当たり判定の描画を行う
		 * @param	bg
		 */
		public function drawHit():void
		{
			if (Debug.isDrawHit() == false) { return; }
			
			for each(var r:HitRect in hitRects)
			{
				_g.setColor(0xFF0000);
				_g.drawRect(x - r.w / 2, y - r.h / 2, r.w, r.h);
			}
		}
		
	}

}