package framework.util 
{
	import framework.device.input.GamePad;
	import framework.GuiButton;
	import framework.GuiItemList;
	import framework.util.GuiItem;
	
	/**
	 * アイテムリスト（パッド対応）
	 * @author syun
	 */
	public class GuiItemList2 extends GuiItemList
	{
		private var m_ItemList:Vector.<GuiItem>;
		private var m_Idx:int;
		private var m_Pad:GamePad;
		public function GuiItemList2(cx:Number, cy:Number, dy:Number, width:Number, height:Number) 
		{
			super(cx, cy, dy, width, height);
			m_Pad = _i.getPad(0);
			clear();
		}
		
		/**
		 * 項目を消去する
		 */
		override public function clear():void 
		{
			m_ItemList = new Vector.<GuiItem>();
			setIdx(0);
		}
		
		/**
		 * 項目数を取得する
		 * @return
		 */
		override public function size():int 
		{
			return m_ItemList.length;
		}
		
		/**
		 * 選択位置を指定する
		 * @param	idx
		 */
		public function setIdx(idx:int):void
		{
			m_Idx = idx;
		}
		
		/**
		 * 選択中の項目を更新する
		 */
		private function updateSelected():void
		{
			for each(var item:GuiItem in m_ItemList)
			{
				item.setSelected(false);
			}
			m_ItemList[m_Idx].setSelected(true);
		}
		
		/**
		 * 選択位置を取得する
		 * @return 選択位置
		 */
		override public function idx():int 
		{
			return m_Idx;
		}
		
		/**
		 * 項目を追加する
		 * @param	name 項目名
		 * @param	bEnable 有効かどうか
		 */
		override public function addItem(name:String, bEnable:Boolean=true):void 
		{
			var px:Number = cx - width / 2;
			var py:Number = cy - dy / 2 + dy * size();
			var item:GuiItem = new GuiItem(name, px, py, width, height);
			item.setEnable(bEnable);
			m_ItemList.push(item);
		}
		
		/**
		 * 選択中の項目を決定できるかどうか
		 * @return 決定できれば「true」
		 */
		public function isDecide():Boolean
		{
			var i:int = 0;
			for each(var item:GuiItem in m_ItemList)
			{
				if (i == idx())
				{
					return item.isEnable();
				}
				i++;
			}
			return false;
		}
		
		/**
		 * 更新
		 */
		override public function update():void 
		{
			// キー入力の更新
			if (m_Pad.isPushDown())
			{
				setIdx(m_Idx + 1);
				if (m_Idx >= size())
				{
					setIdx(0);
				}
			}
			if (m_Pad.isPushUp())
			{
				setIdx(m_Idx - 1);
				if (m_Idx < 0)
				{
					setIdx(size() - 1);
				}
			}
			
			// 選択項目の更新
			updateSelected();
			
			for each(var item:GuiItem in m_ItemList)
			{
				item.update();
			}
		}
		
		/**
		 * 描画
		 */
		override public function draw():void 
		{
			// 枠の描画
			var px:Number = cx - width / 2;
			var py:Number = cy - dy / 2;
			var w:Number = width;
			var h:Number = dy * size();
			_g.setColor(0x0000FF, 0.4);
			_g.fillRect(px-2, py-2, w+4, h+4);
			_g.setColor(0xA000FF, 0.4);
			_g.fillRect(px, py, w, h);
			
			// 項目の描画
			for each(var item:GuiItem in m_ItemList)
			{
				item.draw();
			}
		}
	}

}