package framework.util 
{
	import flash.utils.ByteArray;
	import framework.Maths;
	/**
	 * ２次元マップクラス
	 * @author syun
	 */
	public class Layer2D
	{
		public static const OUT:int = -1;
		private var m_Default:int = 0; // デフォルト値
		private var m_OutOfRange:int = OUT; // 範囲外を指定した際のエラー値
		private var m_Width:int;
		private var m_Height:int;
		private var _pool:Object = { };
		
		/**
		 * コンストラクタ
		 * @param	w 幅
		 * @param	h 高さ
		 */
		public function Layer2D(w:int, h:int) 
		{
			initialize(w, h);
		}
		public function width():int
		{
			return m_Width;
		}
		public function height():int
		{
			return m_Height;
		}
		
		/**
		 * 初期化
		 * @param	w 幅
		 * @param	h 高さ
		 */
		public function initialize(w:int, h:int):void
		{
			_pool = { };
			m_Width = w;
			m_Height = h;
		}
		
		/**
		 * デフォルト値を指定
		 * @param	v
		 */
		public function setDefault(v:int):void
		{
			m_Default = v;
		}
		
		/**
		 * 特定の値で埋める
		 * @param	v 埋める値
		 */
		public function fill(v:int):void
		{
			for (var i:int = 0; i < m_Width * m_Height; i++)
			{
				_pool[i] = v;
			}
		}
		
		/**
		 * ランダムにaの値で埋める
		 * @param	a 埋める値
		 * @param	count 試行回数
		 */
		public function random(a:int, count:int):void
		{
			for (var i:int = 0; i < count; i++)
			{
				var x:int = Maths.rand(m_Width);
				var y:int = Maths.rand(m_Height);
				set(x, y, a);
			}
		}
		/**
		 * a～bの間でランダムな値を設定する
		 * @param	a
		 * @param	b
		 * @param count 試行回数
		 */
		public function random2(a:int, b:int, count:int):void
		{
			for (var i:int = 0; i < count; i++)
			{
				var v:int = Maths.randInt(a, b);
				var x:int = Maths.rand(m_Width);
				var y:int = Maths.rand(m_Height);
				set(x, y, v);
			}
		}
		
		public function copy(layer:Layer2D):void
		{
			layer.initialize(m_Width, m_Height);
			for (var j:int = 0; j < m_Height; j++)
			{
				for (var i:int = 0; i < m_Width; i++)
				{
					var v:int = get(i, j);
					if (v != m_Default)
					{
						layer.set(i, j, v);
					}
				}
			}
		}
		
		/**
		 * 有効な範囲かどうかチェックする
		 * @param	x
		 * @param	y
		 * @return
		 */
		public function check(x:int, y:int):Boolean
		{
			if (x < 0) { return false; }
			if (x >= m_Width) { return false; }
			if (y < 0) { return false; }
			if (y >= m_Height) { return false; }
			return true;
		}
		
		/**
		 * 特定の値が占める割合を取得する
		 * @param	v 対象となる値
		 * @return 0～1の値
		 */
		public function ratio(v:int):Number
		{
			var cnt:int = 0;
			for (var y:int = 0; y < m_Height; y++)
			{
				for (var x:int = 0; x < m_Width; x++)
				{
					var a:int = get(x, y);
					if (a == v)
					{
						cnt++;
					}
				}
			}
			
			return (1.0 * cnt) / (m_Width * m_Height);
		}
		
		/**
		 * (x,y)の指定を一次元のインデックスに変換する
		 * @param	x
		 * @param	y
		 * @return
		 */
		public function getIdx(x:int, y:int):int
		{
			return x + y * m_Width;
		}
		public function get(x:int, y:int):int
		{
			if (check(x, y) == false)
			{
				// 範囲外
				return m_OutOfRange;
			}
			
			var idx:int = getIdx(x, y);
			if (_pool.hasOwnProperty(idx))
			{
				return _pool[idx];
			}
			return m_Default = 0;
		}
		public function set(x:int, y:int, val:int):void
		{
			if (check(x, y) == false) { return; }
			var idx:int = getIdx(x, y);
			_pool[idx] = val;
		}
		
		/**
		 * バイナリデータから設定する
		 * @param	bin
		 */
		public function setByte(bin:ByteArray):void
		{
			for (var y:int = 0; y < m_Height; y++)
			{
				for (var x:int = 0; x < m_Width; x++)
				{
					var v:int = bin.readByte();
					if (v != m_Default)
					{
						set(x, y, v);
					}
				}
			}
		}
		public function dump():void
		{
			for (var j:int = 0; j < m_Height; j++)
			{
				var s:String = "";
				for (var i:int = 0; i < m_Width; i++)
				{
					s += get(i, j);
				}
				trace(s);
			}
		}
	}

}