package framework.util 
{
	import framework.device.graphics.FontObj;
	import framework.device.input.GamePad;
	import framework.IToken;
	/**
	 * メッセージボックス
	 * @author syun
	 */
	public class MessageBox extends IToken
	{
		private static const TIMER_OPEN:int = 30;
		private static const WIDTH:int = 128;
		private static const HEIGHT:int = 48;
		
		// メッセージボックスの種類(m_Type)
		public static const eType_Ok:int = 1; // OKダイアログ
		public static const eType_YesNo:int = 2; // Yes/Noダイアログ
		// 項目番号
		public static const eYesNo_Yes:int = 0;
		public static const eYesNo_No:int = 1;
		
		
		private static var m_Type:int;
		private static var m_Idx:int; // 選択中の項目番号
		private static var m_bActive:Boolean = false;
		private static var m_Timer:int;
		private static var m_Message:String;
		private static var m_Width:int;
		private static var m_Height:int;
		private static var m_Pad:GamePad;
		private static var m_Font:FontObj = null;
		private static var m_FontYes:FontObj = null;
		private static var m_FontNo:FontObj = null;
		
		/**
		 * コンストラクタ
		 */
		public function MessageBox() 
		{
		}
		
		/**
		 * FontObjを使う
		 */
		public static function useFontObj():void
		{
			m_Font = new FontObj();
			m_Font.setFormat(0xFFFFFF, 14, false, true);
			m_FontYes = new FontObj();
			m_FontYes.setFormat(0xFFFFFF, 14, false, true);
			m_FontYes.setText("はい");
			m_FontNo = new FontObj();
			m_FontNo.setFormat(0xFFFFFF, 14, false, true);
			m_FontNo.setText("いいえ");
		}
		/**
		 * メッセージボックスを開く
		 * @param	msg メッセージ文字列
		 * @param type メッセージボックスの種類
		 * @param idx カーソル位置
		 */
		public static function open(msg:String, type:int=eType_Ok, idx:int=0):void
		{
			if (m_Font)
			{
				m_Font.setText(msg);
			}
			else
			{
				m_Message = msg;
			}
			m_Type = type;
			m_Idx = idx;
			m_bActive = true;
			m_Timer = TIMER_OPEN;
			m_Width = WIDTH;
			m_Height = HEIGHT;
			m_Pad = _i.getPad(0);
			_m.play("push");
		}
		
		/**
		 * 選択した項目番号を取得する
		 * @return 項目番号
		 */
		public static function idx():int
		{
			return m_Idx;
		}
		
		/**
		 * メッセージボックスを開いているかどうか
		 * @return
		 */
		public static function isOpen():Boolean
		{
			return m_bActive;
		}
		/**
		 * メッセージボックスを閉じる
		 */
		public static function close():void
		{
			m_bActive = false;
		}
		
		/**
		 * 更新
		 */
		public static function update():void
		{
			if (m_bActive == false) { return; }
			
			if (m_Timer > 0)
			{
				m_Timer = m_Timer * 50 / 100;
				return;
			}
			if(m_Pad.isPushEnter())
			{
				_m.play("pi");
				close();
			}
			switch(m_Type)
			{
			case eType_YesNo:
				if (m_Pad.isPushArrawAny())
				{
					m_Idx = (m_Idx == eYesNo_Yes ? eYesNo_No : eYesNo_Yes);
				}
				break;
			}
		}
		
		/**
		 * 描画
		 */
		public static function draw():void
		{
			if (m_bActive == false) { return; }
			
			// 中心座標を取得する
			var cx:Number = Main.WIDTH / 2;
			var cy:Number = Main.HEIGHT / 2;
			
			// ウィンドウを描画
			var h:Number = m_Height * (TIMER_OPEN - m_Timer) / TIMER_OPEN;
			_g.setColor(0xFFFFFF, 0.8);
			_g.fillRectEx(cx, cy, m_Width+2, h+2);
			_g.setColor(0x202040, 0.8);
			_g.fillRectEx(cx, cy, m_Width, h);
			
			// メッセージを描画
			if (m_Timer == 0)
			{
				if (m_Font)
				{
					draw2(cx-48, cy-16, m_Idx);
				}
				else
				{
					_f.drawAsciiEx(m_Message, cx, cy - 16);
					switch(m_Type)
					{
					case eType_Ok:
						_f.drawAsciiEx("OK", cx, cy + 24);
						break;
					case eType_YesNo:
						_f.drawAsciiEx("YES", cx - 24, cy + 24, m_Idx == eYesNo_Yes ? 1.5 : 1);
						_f.drawAsciiEx("NO", cx + 24, cy + 24, m_Idx == eYesNo_No ? 1.5 : 1);
						break;
					}
				}
			}
		}
		
		/**
		 * FontObjを使って描画する
		 * @param	cx
		 * @param	cy
		 * @param	idx
		 */
		private static function draw2(cx:Number, cy:Number, idx:int):void
		{
			m_Font.draw2(cx, cy - 16);
			switch(m_Type)
			{
			case eType_YesNo:
				_g.setColor(0xFFFFFF);
				if (m_Idx == eYesNo_Yes)
				{
					_g.drawRect(cx - 24+24, cy + 24, 40, 16);
				}
				else
				{
					_g.drawRect(cx + 24+24, cy + 24, 40, 16);
				}
				m_FontYes.draw2(cx - 28, cy + 22);
				m_FontNo.draw2(cx + 20, cy + 22);
				break;
			}
		}
	}

}