package scene 
{
	import flash.utils.ByteArray;
	import framework.device.input.GamePad;
	import framework.FMFLoader;
	import framework.resource.ResourceFmf;
	import framework.scene.IScene;
	import framework.util.Layer2D;
	import util.View;
	
	/**
	 * タイトル画面クラス
	 * @author syun
	 */
	public class SceneTitle extends IScene
	{
		/**
		 * マップ描画オフセット(X)
		 */
		public var MAP_OFSX:Number = 160;
		
		/**
		 * マップ描画オフセット(Y)
		 */
		public var MAP_OFSY:Number = 120;
		
		/**
		 * チップ描画オフセット(X)
		 */
//		public var CHIP_OFSX:Number = 30;
		public var CHIP_OFSX:Number = 20;
		
		/**
		 * チップ描画オフセット(Y)
		 */
//		public var CHIP_OFSY:Number = 15;
		public var CHIP_OFSY:Number = 10;

		/**
		 * チップサイズ(幅)
		 */
		public var CHIP_WIDTH:Number = 34;
		
		/**
		 * チップサイズ(高さ)
		 */
		public var CHIP_HEIGHT:Number = 17;
		
		/**
		 * マップチップデータの１行あたりの数
		 */
		public var CHIP_COL:int = 8;
		
		/**
		 * パッド入力管理
		 */
		private var m_Pad:GamePad;
		
		/**
		 * マップ情報
		 */
		private var m_Fmf:FMFLoader;
		
		/**
		 * マップ情報(マップチップ)
		 */
		private var m_Layer:Layer2D;
		
		/**
		 * 背景更新タイマー
		 */
		private var m_Timer:int = 0;
		
		/**
		 * 選択中のマップチップ = 視点の中心
		 */
		private var m_SelIdx:int;
		private var m_CenterIdx:int;
		
		/**
		 * 視点管理クラス
		 */
		private var m_View:View;
		
		/**
		 * コンストラクタ
		 */
		public function SceneTitle() 
		{
			m_Pad = _i.getPad(0);
			m_View = new View();
		}
		
		/**
		 * 初期化
		 */
		override public function initialize():void 
		{
			// マップデータ読み込み
			var b:ByteArray = ResourceFmf.getValue("01");
			b.position = 0;
			m_Fmf = new FMFLoader(b);
			m_Layer = m_Fmf.getLayer(0);
			//m_Layer.dump();
			
			// マップの中心に移動する
			m_SelIdx = m_Layer.width() / 2 + (m_Layer.height() / 2 * m_Layer.width());
			m_CenterIdx = m_SelIdx;
			startScroll();
		}
		
		/**
		 * スクロールを開始する
		 */
		public function startScroll():void
		{
			// マップ座標 -> クォータービュー座標 に変換
			// X = w(x + y)
			// Y = h(x - y)
			var cx:int = m_CenterIdx % m_Layer.width();
			var cy:int = m_CenterIdx / m_Layer.width();
			var nextX:Number = CHIP_OFSX * (cx + cy);
			var nextY:Number = CHIP_OFSY * (cx - cy);
			
			// スクロール開始
			m_View.start(nextX, nextY);
		}
		
		/**
		 * 更新
		 */
		override public function update():void 
		{
			// マウス座標を取得
			var x:Number = m_Pad.x();
			var y:Number = m_Pad.y();
			m_SelIdx = quarterToMap(x, y);
			if (m_Pad.isPushEnter())
			{
				// 画面の中心を移動
				if (m_SelIdx >= 0)
				{
					m_CenterIdx = m_SelIdx;
					startScroll();
				}
			}
			
			// 視点更新
			m_View.update();
		}
		
		/**
		 * クォータービュー座標をマップ座標に変換
		 * @param	x クォータービュー座標(X)
		 * @param	y クォータービュー座標(Y)
		 * @return マップ座標インデックス
		 */
		public function quarterToMap(x:int, y:int):int
		{
			var px:Number = x;
			var py:Number = y;
			
			// カメラ座標を反映
			px += m_View.getX();
			py -= m_View.getY();
			
			// マップ全体のオフセットを反映
			px = (px - MAP_OFSX);
			py = -(py - MAP_OFSY);
			
			// チップのオフセットをする(クォータービュー座標は「左中心」がデカルト座標の「左上」)
			px += CHIP_WIDTH / 2;
			
			// クォータービュー座標 -> マップ座標 へ変換
			// x = X/2w + Y/2h
			// y = X/2w + Y/2h
			var i:int = Math.floor((px / CHIP_OFSX / 2) + (py / CHIP_OFSY / 2));
			var j:int = Math.floor((px / CHIP_OFSX / 2) - (py / CHIP_OFSY / 2));
			
			// 範囲外チェック
			if (i < 0 || m_Layer.width() <= i) { return -1; }
			if (j < 0 || m_Layer.height() <= j) { return -1; }
			
			// マップ用インデックスに変換
			return j * m_Layer.width() + i;;
		}
		
		/**
		 * 描画
		 */
		override public function draw():void 
		{
			// 背景
			m_Timer += 5;
			_g.setColor(0x202020);
			_g.drawTriangleEx(160, 120, 128, m_Timer);
			_g.drawTriangleEx(160, 120, 128, m_Timer+180);
			
			// チップの描画
			for (var j:int = 0; j < m_Layer.height(); j++)
			{
				for (var i:int = m_Layer.width()-1; i >= 0 ; i--)
				{
					if (j * m_Layer.width() + i == m_SelIdx)
					{
						// 選択中のチップは描画しない
						continue;
					}
					
					var v:int = m_Layer.get(i, j);
					if (v != 0)
					{
						drawChip(v, i, j);
					}
				}
			}
			
			// デバッグ用情報
			_f.drawAscii("x=" + m_Pad.x(), 0, 0);
			_f.drawAscii("y=" + m_Pad.y(), 0, 10);
			_f.drawAscii("idx=" + m_CenterIdx, 0, 20);
		}
		
		/**
		 * チップの描画 (クォータービュー座標に変換))
		 * @param	id チップ番号
		 * @param	i マップ座標(X)
		 * @param	j マップ座標(Y)
		 */
		private function drawChip(id:int, i:int, j:int):void
		{
			// マップ座標 -> クォータービュー座標 に変換
			// X = w(x + y)
			// Y = h(x - y)
			var x:Number = CHIP_OFSX * (i + j);
			var y:Number = CHIP_OFSY * (i - j);
			
			// 表示オフセットを反映
			x = x + MAP_OFSX;
			y = -y + MAP_OFSY;
			
			// スクロールを反映
			if (m_CenterIdx >= 0)
			{
				x -= m_View.getX();
				y += m_View.getY();
			}
			
			// チップの描画
			var ox:Number = CHIP_WIDTH * (id%CHIP_COL);
			var oy:Number = CHIP_HEIGHT * (Math.floor(id / CHIP_COL));
			_g.drawFastEx("chip", x, y, ox, oy, CHIP_WIDTH, CHIP_HEIGHT);
//			_f.drawAsciiEx("" + id, x, y, 4);
		}
	}

}